/* 
 * Cliente restful para sincronizar con el servidor.
 */

var restful = {
	userID: null,
	
	init: function(email) {
		if (global.debug) console.log("restful init function");
		restful.userID = hex_md5(email + global.SALT);
	},
	
	// action: 'POST' (insert) o 'PUT' (update)
	user: function(action, email, password) {
		if (global.debug) console.log("restful user function");
		restful.init(email);
		var message = new Object();
		message['email'] = email;
		message['password'] = password;
		if (global.debug) console.log("restful user message: "+JSON.stringify(message));
		
		if (action == 'POST') {
			if (global.debug) console.log("restful user function: POST");
			var url = '/api/user';
		} else {
			if (global.debug) console.log("restful user function: PUT");
			var url = '/api/user/' + restful.userID;
		}
		
		restful.ajaxCall(action, url, JSON.stringify(message));
	},
	
	inr: function(action, valor, fecha) {
		if (global.debug) console.log("restful inr function");
		var message = new Object();
		message['valor'] = valor;
		message['fecha'] = fecha;
		var url = '/api/inr/' + restful.userID;
		
		if (dao.isUpToDate()) {
			restful.ajaxCall(action, url, JSON.stringify(message));
		} else {
			restful.saveAjaxCall(action, url, JSON.stringify(message));
		}
	},
	
	dosis: function(action, valor, fecha) {
		if (global.debug) console.log("restful dosis function");
		var message = new Object();
		message['valor'] = valor;
		message['fecha'] = fecha;
		var url = '/api/dosis/' + restful.userID;
		
		if (dao.isUpToDate()) {
			restful.ajaxCall(action, url, JSON.stringify(message));
		} else {			
			restful.saveAjaxCall(action, url, JSON.stringify(message));
		}
	},
	
	ajaxCall: function(action, callUrl, message, live) {
		live = typeof live !== 'undefined' ? live : true;
		if (global.debug) console.log("ajaxCall action: "+ action);
		if (global.debug) console.log("ajaxCall url: "+ global.SERVER_URL + callUrl);
		if (global.debug) console.log("ajaxCall message: "+ message);
		jQuery.ajax({
			type: action,
			url: global.SERVER_URL + callUrl, 
			data: message,
			contentType: "application/json; charset=utf-8",
			dataType: "json",
			timeout: global.SERVER_TIMEOUT,
			success: function (data, status, jqXHR) {
				if (global.debug) console.log(action+" success: "+ status);
				if (data.result == 'success') {
					if (!live) {
						// Si era una peticion ajax almacenada en la bbdd que se ha ejecutado
						// con éxito, debemos eliminarla para no volverla a ejecutar.
						app.db.transaction(dao.deleteAjaxCall, dao.errorCB);
						// Como el servidor esta funcionando, seguimos enviando peticiones
						// hasta que no nos quede ninguna almacenada.
						setTimeout(function() { app.db.transaction(dao.checkServer, dao.errorCB) }, global.SERVER_RECONNECT);
					}
				} else {
					if (data.code == '1') {
						if (global.debug) console.log("Usuario ya existe, password correcto, accedemos.");
						app.clickEvent('#valorinr');
					} else if (data.code == '2') {
						if (global.debug) console.log("Usuario ya existe, password incorrecto, no accedemos.");
						navigator.notification.alert(
							'El correo electrónico introducido ya está en uso, debe escoger otro.',
							null,         								// callback
							'Aviso',            						// title
							'Aceptar'									// buttonName
						);
						restful.goToInicio();
					}				
					restful.parseError(data);
					$.unblockUI();
				}				
				dao.setLastSync(new Date());
			},
     
			fail: function (jqXHR, status) {            
				if (global.debug) console.log(action+" error: "+ status);
				restful.manageError(action, callUrl, message, live);
			},
			
			statusCode: {
				404: function() {
					if (global.debug) console.log("statusCode 404");
					restful.manageError(action, callUrl, message, live);
				},
				500: function() {
					if (global.debug) console.log("statusCode 500");
					restful.manageError(action, callUrl, message, live);
				},
				0: function() {
					if (global.debug) console.log("statusCode 0");
					restful.manageError(action, callUrl, message, live);
				},
			},

		});
	},
	
	manageError: function(action, callUrl, message, live) {
		if (global.debug) console.log("restful.manageError function. callUrl: "+ callUrl);
		if (callUrl.indexOf('user') != -1) {
			if (global.debug) console.log("manage user error");
			navigator.notification.alert(
				'Error al conectar con el servidor, por favor, inténtelo más tarde.',
				null,         								// callback
				'Aviso',            						// title
				'Aceptar'									// buttonName
			);
			restful.goToInicio();
			$.unblockUI();
			
		} else if (callUrl.indexOf('user') == -1) {
			if (live) {
				restful.saveAjaxCall(action, callUrl, message);
			}
			dao.setUpToDate(0);
			setTimeout(function() { app.db.transaction(dao.checkServer, dao.errorCB) }, global.SERVER_RECONNECT);
		}		
	},
	
	parseError: function(data) {
		if (data['result'] == 'error') {
			if (global.debug) console.log("errorMessage: "+ data['message']);
		} else {
			if (global.debug) console.log("Unknow error: "+ JSON.stringify(data));
		}
	},
	
	saveAjaxCall: function(action, url, message) {
		if (global.debug) console.log("saveAjaxCall function");
		global.ajaxCall = new Object;
		global.ajaxCall.action = action;
		global.ajaxCall.callUrl = url;
		global.ajaxCall.message = message;
		app.db.transaction(dao.saveAjaxCall, dao.errorCB);
	},
	
	goToInicio: function() {
		if (global.debug) console.log("goToInicio function");
		$('#email').val('');
		dao.saveEmail(null);
		$('#contrasenya').val('');
		dao.savePassword(null);
		app.clickEvent('#inicio');
	}
};